import 'dart:core';

/// Use this class to validate password,number,name ,email and special characters
class Validator {
  static const int passwordMin = 6;
  static const int passwordMax = 32;
  static final RegExp passwordPattern = RegExp(
      '(((?=.*\\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#&%]).{' +
          '$passwordMin,$passwordMax}))');
  static final RegExp numberPattern = RegExp(r'\d');
  static final RegExp namePattern = RegExp(r"[a-zA-Z0-9 ]*");
  static final RegExp specialCharactersPattern = RegExp(r"[$=\\\\|<>^*%]");
  static final RegExp emailPattern = RegExp(
      r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$');

  static bool isValidPassword(String password) {
//    return password?.contains(passwordPattern, 0);
    return password != null && passwordPattern.hasMatch(password);
  }

  static bool isNameValid(String name) {
    return name != null && namePattern.hasMatch(name);
  }

  static bool isEmailValid(String email) {
    return email != null && emailPattern.hasMatch(email);
  }
}

/// Errors validator
class KeyValueValidator {
  Map<String, String> _values = {};
  Map<String, String> _errors = {};

  addValue(String key, String value) {
    if (key != null) _values[key] = value;
  }

  addValues(Map<String, dynamic> keyValues) {
    if (keyValues != null) {
      keyValues.forEach((key, value) {
        if (value == null || value.runtimeType == String) {
          addValue(key, value);
        } else {
          removeValue(key);
        }
      });
    }
  }

  addError(String key, String error) {
    if (key != null) _errors[key] = error;
  }

  addErrors(Map<String, String> keyValues) {
    if (keyValues != null) _errors.addAll(keyValues);
  }

  removeValue(String key) {
    if (key != null) _values.remove(key);
  }

  removeError(String key) {
    if (key != null) _errors.remove(key);
  }

  clearValues() {
    _values.clear();
  }

  clearErrors() {
    _errors.clear();
  }

  bool containsValue(String key) {
    return _values.containsKey(key);
  }

  bool containsError(String key) {
    return _errors.containsKey(key);
  }

  String getValue(String key) {
    if (key == null) return null;
    return _values[key];
  }

  String getError(String key) {
    if (key == null) return null;
    return _errors[key];
  }

  int errorsLength() {
    return _errors.length;
  }

  int valuesLength() {
    return _values.length;
  }

  bool isEmpty(String str) {
    return str == null || str.trim().isEmpty;
  }

  bool checkEmpty(List<String> keys, [String errorMsg = "Required field"]) {
    bool empty = false;
    if (keys != null) {
      keys.forEach((key) {
        if (isEmpty(getValue(key))) {
          empty = true;
          addError(key, errorMsg ?? "Required field");
        }
      });
    }
    return empty;
  }

  bool checkEmail(String key, [String errorMsg = "Invalid email"]) {
    if (!Validator.isEmailValid(getValue(key))) {
      addError(key, errorMsg ?? "Invalid email");
      return false;
    }
    return true;
  }

  bool checkName(String key, [String errorMessage = "Invalid name"]) {
    if (!Validator.isNameValid(getValue(key))) {
      addError(key, errorMessage ?? "Invalid name");
      return false;
    }
    return true;
  }

  bool checkPassword(String key, [String errorMessage = "Invalid password"]) {
    if (!Validator.isValidPassword(getValue(key))) {
      addError(key, errorMessage ?? "Invalid password");
      return false;
    }
    return true;
  }
}
